.include "2313def.inc"

.equ	IND_DATA	=5		;IND_DATA  conn to pin #5
.equ	IND_CLOCK	=6		;IND_CLOCK conn to pin #6

.def	CNT1		=R25		;(GL)Counter for DIGIT_7SEG1/DIGIT_7SEG2 Select

.def	KEY		=R24		;(GL)KEY Codes

.def	ACK		=R23		;(GL)ACK Code		

.def	DEL25		=R1		;(GL)DELAY 0.25 ms

.def	ALRM		=R2		;(GL)ALRM On/Off Flag

.def	POINT		=R4		;(GL)POINT On/Off Flag

;======================== *** SRAM SEGMENT ***==============================

	.DSEG  				;SRAM memory segment


IND_COUNT:	.byte	1		;IND Counter
IND_ADR:	.byte	1		;IND Adress
IND_FLAG:	.byte	1		;IND FLAG (DIGIT_7SEG1/DIGIT_7SEG2 Select)

KEY_SHIFT:	.byte	1		;KEY CODES CREATING

WR_TIME_FL:	.byte	1		;"0" - write TIME; "1" - write ALARM

DIGIT_7SEG1:	.byte	4		;DIGIT in 7-SEG Code 1
DIGIT_7SEG2:	.byte	4		;DIGIT in 7-SEG Code 2
DIGIT_DEC1:	.byte	4		;DIGIT in DEC Code 1
DIGIT_DEC2:	.byte	4		;DIGIT in DEC Code 2

PCF8583_FLAG:	.byte	1		;PCF8583 WRITE/READ Flag
PCF8583_A00:	.byte	1		;PCF8583 STATUS CONTROL Register MIRROR
PCF8583_A01:	.byte	1		;
PCF8583_A02:	.byte	1		;
PCF8583_A03:	.byte	1		;PCF8583 CLOCK MINUTES  Register MIRROR
PCF8583_A04:	.byte	1		;PCF8583 CLOCK HOURS    Register MIRROR
PCF8583_A05:	.byte	1		;
PCF8583_A06:	.byte	1		;
PCF8583_A07:	.byte	1		;
PCF8583_A08:	.byte	1		;PCF8583 ALARM CONTROL  Register MIRROR
PCF8583_A09:	.byte	1		;
PCF8583_A0A:	.byte	1		;
PCF8583_A0B:	.byte	1		;PCF8583 ALARM MINUTES  Register MIRROR
PCF8583_A0C:	.byte	1		;PCF8583 ALARM HOURS    Register MIRROR
PCF8583_A0D:	.byte	1		;
PCF8583_A0E:	.byte	1		;
PCF8583_A0F:	.byte	1		;

;======================== *** CODE SEGMENT ***==============================

	.CSEG     	 		;begin of program memory
	.org 0

;-----------------SETUP INTERRUPTS HANDLERS---------------------------------

reset:	rjmp	start			; 00 Reset Handler
	reti				; 01 IRQ0 Handler
	reti				; 02 IRQ1 Handler
	reti				; 03 Timer1 Capture Handler
	reti				; 04 Timer1 Compare Handler
	rjmp	TC1_OVF			; 05 Timer1 Overflow Handler
	rjmp	TimeKey			; 06 Timer0 Overflow Handler
	reti				; 07 UART RX Complete Handler
	reti				; 08 UDR Empty Handler
	reti				; 09 UART TX Complete Handler
	reti				; 0A Analog Comparator Handler
	
;--------------------- Messages & Constants --------------------------------

;    NAMES:    "0"  "1"  "2"  "3"  "4"  "5"  "6"  "7"  "8"  "9"  "A"  "B"	

DIGITS:	.db	0xDB,0x48,0xE3,0xEA,0x78,0xBA,0xBB,0xC8,0xFB,0xFA,0x00,0xE7

;ORDER NUM:    "0"  "1"  "2"  "3"  "4"  "5"  "6"  "7"  "8"  "9"  "-"  "2."

VERSION:.db	0x0A,0x0A,0x0A,0x0A,0x01,0x08,0x0B,0x00,0x01,0x0A,0x0A,0x0A,0x0A,0x0A
;----------------------- BEGIN OF PROGRAMM ---------------------------------

;---------------------- *** setup part *** ---------------------------------

start:	
	
	ldi	r16,0xdf		;stack init SP=0xdf
	out	spl,r16			;

	sbi	ACSR,ACD		;Analog comparator off

	ldi	r16,0xFF		;Set Ini Ports func (DIRECTION)
	out	ddrb,r16		;	DDRB

	ldi	r16,0x00		;Set Ini Ports value (VALUE)
	out	portb,r16		;	PORTB

	ldi	r16,0b01101000		;Set Ini Ports func (DIRECTIO)
	out	ddrd,r16		;	DDRD

	ldi	r16,0b00010100		;Set Ini Ports value (VALUE)
	out	portd,r16		;	PORTD

	ldi	r16,0b00000100
	out	TCCR0,r16		;T/C0 CK/256
	ldi	r16,0b00000010
	out	TIMSK,r16		;T/C0 Overflow INT enable
	ldi	r16,177
	out	TCNT0,r16		;T/C0 start from 177 (at 4MHz 200Hz- digit, 50Hz- 4 digits)

	ldi	r16,0b00000001
	out	TCCR1B,r16		;T/C1 T1 (CK)
	ldi	r16,0b10000010
	out	TIMSK,r16		;T/C1 Overflow INT enable
	ldi	r16,0xFC		;
	out	TCNT1H,r16		; 
	ldi	r16,0x17		;
	out	TCNT1L,r16		;TC1 Start from 0xFC17 (2KHz)
	
	;ldi	r16,0b01000000
	;out	GIMSK,r16		;INT0 enable
	;ldi	r16,0b00000010
	;out	MCUCR,r16		;INT0 enable on FALLING EDGE

;--------------------- Registers clearing ----------------------------------

	ldi	r31,0x00		;Load 0=>MSB (Z)
	ldi	r30,0x1d		;Load 29=>LSB (Z)
RgClr:	st	Z,R31			;0=>R(Z)
	dec	R30			;Counter=counter-1
	brne	RgClr			;Do until counter=0

;--------------------- RAM/PCF8583 clearing & Organaizing  -----------------

	ldi	r16,4
	sts	IND_COUNT,r16
	clr	r16

	sts	PCF8583_FLAG,r16

;---------------------  ALARM OFF after POWER Break	------------------

		
	;rcall	I2CSTART		;BEGIN OF TRANSFER
	;ldi	r16,0xA0		;RD Addr PLL => I2C
	;rcall	I2COUT			;Exequte 
	;ldi	r16,0x08		;To 0x08 ADR (ALARM status rgister)
	;rcall	I2COUT			;Exequte

	;clr	r16,r17			;GET Digit from RAM
	;rcall	I2COUT			;WRITE Byte=>R16 
	
	;rcall	I2CSTOP			;END OF TRANSFER


;--------------------- MASTER RESET (ANY KEY On POWER UP) ------------------
	
	in	r16,PIND		;Read BUTTONS state
	swap	r16
	andi	r16,0b00000001		;Get BUTTON Pressed/Not pressed

	tst	r16
	brne	MR1

	ldi	ACK,15			;ACK --> Counter=15
	ldi	XL,PCF8583_A00		;XL --> First ADRESS in RAM
	clr	r16

MR2:	st	X+,R16			;Store REGISTER to RAM
	dec	ACK			;DEC ACK (Counter)
	brne MR2

	ser	r16			;
	sts	PCF8583_FLAG,r16	;Enable WRITE SICLE
MR1:

;---------------------- Main program ---------------------------------------
		
	sei			;global INT enable	
	
;---------------------- VERSION Indication ---------------------------------	

		
		ldi	r18,0			;Counter=0...13
		ser	r19			;
		mov	POINT,r19		;Point OFF
			
VR2:		
		ldi	YL,DIGIT_DEC2+4
		ldi	XL,DIGIT_DEC1+4
		ldi	r17,4			;Counter=0...4
		ldi	ZL,VERSION*2		;First adress of digits codes
		add	ZL,r18

VR1:		LPM				;LPM from ZL to R0
		ST	-Y,R0
		ST	-X,R0
		inc	ZL		
		dec	r17
		brne	VR1
		
		rcall	DL25
		rcall	DL25
		rcall	DL25
		
		inc	r18
		cpi	r18,11
		brne	VR2

		clr	POINT			;POINT ON
		
;---------------------------------------------------------------------------
	
LOOP:		
		clr	KEY
		
		lds	r16,PCF8583_A08	;
		sbrs	r16,4			;
		rjmp	NEXT1			;
		ser	R16			;
		mov	ALRM,R16		;
		rjmp	NEXT2			;
NEXT1:		clr	ALRM			;Resume ALARM after POWER Break
NEXT2:

		lds	r16,PCF8583_A04	;4

		mov	r17,r16			;
		swap	r17			;
		andi	r17,0b00001111		;
		andi	r16,0b00001111		;
		sts	DIGIT_DEC1+3,R17	;
		sts	DIGIT_DEC1+2,R16	;
		sts	DIGIT_DEC2+3,R17	;
		sts	DIGIT_DEC2+2,R16	;Display TIME HOURS		

		lds	r16,PCF8583_A03	;3
	
		mov	r17,r16			;
		swap	r17			;
		andi	r17,0b00001111		;
		andi	r16,0b00001111		;
		sts	DIGIT_DEC1+1,R17	;
		sts	DIGIT_DEC1,R16		;
		sts	DIGIT_DEC2+1,R17	;
		sts	DIGIT_DEC2,R16		;Display TIME MINUTES

		tst	KEY			;
		breq	LOOP			;In any KEY not pressed goto LOOP
	
		sbrc	KEY,3
		rjmp	SET_ALARM
		sbrc	KEY,2
		rjmp	SET_TIM
		sbrc	KEY,1
		rjmp	SHOW_SEC
		sbrc	KEY,0
		rjmp	ALARM_ONOFF

		clr	KEY
		rjmp	LOOP

;--------------------- SHOW SEC/MIN ROUTINE ----------------------------------

SHOW_SEC:	rcall	DL25
SHOW_SEC1:	clr	KEY

		lds	r16,PCF8583_A03	;

		mov	r17,r16			;
		swap	r17			;
		andi	r17,0b00001111		;
		andi	r16,0b00001111		;
		sts	DIGIT_DEC1+3,R17	;
		sts	DIGIT_DEC1+2,R16	;
		sts	DIGIT_DEC2+3,R17	;
		sts	DIGIT_DEC2+2,R16	;Display MINUTES		

		lds	r16,PCF8583_A02	;
	
		mov	r17,r16			;
		swap	r17			;
		andi	r17,0b00001111		;
		andi	r16,0b00001111		;
		sts	DIGIT_DEC1+1,R17	;
		sts	DIGIT_DEC1,R16		;
		sts	DIGIT_DEC2+1,R17	;
		sts	DIGIT_DEC2,R16		;Display SECONDS	

		tst	KEY			;
		breq	SHOW_SEC1		;In any KEY not pressed goto SHOW_SEC
	
		rcall	DL25
		
		rjmp	LOOP

;--------------------- SET TIME ROUTINE ----------------------------------
SET_TIM:	clr	r16
		sts	WR_TIME_FL,r16		;Enable WRITE TIME SICLE

SET_TIME:	rcall	DL25

SET_TIME1:	clr	KEY
		
		rcall	BLINK_H			;Blink HOURS

		tst	KEY			;
		breq	SET_TIME1		;If any KEY not pressed goto SET_TIME
	
		sbrc	KEY,2
		rjmp	H24_TEST
		sbrc	KEY,1
		rjmp	HMSB_UP
		sbrc	KEY,0
		rjmp	HLSB_UP

		rjmp	SET_TIME1

HMSB_UP:	lds	r16,DIGIT_DEC2+3
		inc	r16
		cpi	r16,3
		brne	HMSB_UP1
		ldi	r16,0x00
HMSB_UP1:	sts	DIGIT_DEC2+3,r16
		rjmp	SET_TIME


HLSB_UP:	lds	r16,DIGIT_DEC2+2
		inc	r16
		cpi	r16,10
		brne	HLSB_UP1
		ldi	r16,0x00
HLSB_UP1:	sts	DIGIT_DEC2+2,r16
		rjmp	SET_TIME

H24_TEST:	lds	r16,DIGIT_DEC2+3
		cpi	r16,2
		brne	SET_MINUTES

		lds	r16,DIGIT_DEC2+2
		cpi	r16,5
		brsh	SET_TIME


SET_MINUTES:	rcall	DL25
SET_MINUTES1:	clr	KEY
		
		lds	r16,DIGIT_DEC2+3	;
		sts	DIGIT_DEC1+3,r16	;
		lds	r16,DIGIT_DEC2+2	;
		sts	DIGIT_DEC1+2,r16	;Recover HOURS

		rcall	BLINK_M			;Blink MINUTES

		tst	KEY			;
		breq	SET_MINUTES1		;If any KEY not pressed goto SET_MINUTES
	
		sbrc	KEY,2
		rjmp	WRITE_TIME
		sbrc	KEY,1
		rjmp	MMSB_UP
		sbrc	KEY,0
		rjmp	MLSB_UP

		rjmp	SET_MINUTES1

MMSB_UP:	lds	r16,DIGIT_DEC2+1
		inc	r16
		cpi	r16,6
		brne	MMSB_UP1
		ldi	r16,0x00
MMSB_UP1:	sts	DIGIT_DEC2+1,r16
		rjmp	SET_MINUTES


MLSB_UP:	lds	r16,DIGIT_DEC2
		inc	r16
		cpi	r16,10
		brne	MLSB_UP1
		ldi	r16,0x00
MLSB_UP1:	sts	DIGIT_DEC2,r16
		rjmp	SET_MINUTES
	

WRITE_TIME:	rcall	DL25
		clr	KEY
		lds	r16,DIGIT_DEC2+3	;Get H_HOURS
		lds	r17,DIGIT_DEC2+2	;Get L_HOURS
		swap	r16
		or	r16,r17			;R16 --> HOURS BYTE

		lds	r17,DIGIT_DEC2+1	;Get H_MINUTES
		lds	r18,DIGIT_DEC2		;Get L_MINUTES
		swap	r17
		or	r17,r18			;R17 --> MINUTES BYTE

		lds	r18,WR_TIME_FL		;Load WRITE TIME FLAG
		tst	r18
		brne	WR_ALARM

		cli				;
		sts	PCF8583_A04,r16	;
		sts	PCF8583_A03,r17	;
		ser	r16			;
		sts	PCF8583_FLAG,r16	;
		sei				;ENABLE TIME WRITE SICLE

		rjmp	LOOP		

WR_ALARM:	cli				;
		sts	PCF8583_A0C,r16	;
		sts	PCF8583_A0B,r17	;
		ser	r16			;
		sts	PCF8583_FLAG,r16	;
		sei				;ENABLE ALARM WRITE SICLE

		rjmp	LOOP

;----------------------- SET ALARM TIME ROUTINE --------------------------

SET_ALARM:	lds	r16,PCF8583_A0C	;

		mov	r17,r16			;
		swap	r17			;
		andi	r17,0b00001111		;
		andi	r16,0b00001111		;
		sts	DIGIT_DEC1+3,R17	;
		sts	DIGIT_DEC1+2,R16	;
		sts	DIGIT_DEC2+3,R17	;
		sts	DIGIT_DEC2+2,R16	;Display ALARM HOURS		

		lds	r16,PCF8583_A0B	;
	
		mov	r17,r16			;
		swap	r17			;
		andi	r17,0b00001111		;
		andi	r16,0b00001111		;
		sts	DIGIT_DEC1+1,R17	;
		sts	DIGIT_DEC1,R16		;
		sts	DIGIT_DEC2+1,R17	;
		sts	DIGIT_DEC2,R16		;Display ALARM MINUTES

		ser	r16
		sts	WR_TIME_FL,r16		;Enable WRITE ALARM SICLE		

		rjmp	SET_TIME		;GET SET_TIME ROUTINE
		

;-----------------------  ALARM ON/OFF ROUTINE --------------------------


ALARM_ONOFF:	rcall	DL25
		clr	KEY	

		cli				;Global INT Disable

		tst	ALRM
		brne	AL1

		ser	R16
		mov	ALRM,R16
		
		ldi	r16,0b00010000		;Setup ALARM register 
		sts	PCF8583_A08,r16
		
		rjmp	AL2
		
AL1:		clr	ALRM
		lds	r16,PCF8583_A00
		andi	r16,0b11111101		;Clear  ALARM flag
		sts	PCF8583_A00,r16

		clr	r16
		sts	PCF8583_A08,r16	;Clear  ALARM ENABLE flag

		cbi	portb,4			;RELE OFF
		cbi	portd,3			;CLEAR PORTD3=1 (BEEP)

AL2:		ser	r16			;
		sts	PCF8583_FLAG,r16	;Enable WRITE SICLE
	
		sei				;Global INT Enable
		
		rjmp	LOOP


;---------------- Blink HOURS ROUTINE ------------------------------------

BLINK_H:
		ldi	r16,0x0A		;
		sts	DIGIT_DEC1+3,R16	;
		sts	DIGIT_DEC1+2,R16	;Blink HOURS

		RET

;---------------- Blink MINUTES ROUTINE ----------------------------------

BLINK_M:
		ldi	r16,0x0A		;
		sts	DIGIT_DEC1+1,R16	;
		sts	DIGIT_DEC1,R16		;Blink MINUTES

		RET

;---------------- DELAY 025s ---------------------------------------------

DL25:		cli
		ldi	r16,50
		mov	DEL25,r16
		sei
DEL1:		tst	DEL25
		brne	DEL1

		RET	

;-------------------------------------------------------------------------


.include "timekey.asm"
.include "DEC27SEG.asm"
.include "I2C.asm"
.include "TC1_OVF.asm"
